@@ -25,7 +25,7 @@ const char *cCriteria::GetChannelName() const {
 
 bool cCriteria::GoodVPS() {
     if (!channelName) return false;
-
+    if (CompareChannelName(channelName, "ARD_alpha",    IGNORE_HD))               return true;
 //    if (CompareChannelName(channelName, "arte",         IGNORE_HD))               return true;    // VPS start event before preview
     if (CompareChannelName(channelName, "Das_Erste",    IGNORE_HD))               return true;
     if (CompareChannelName(channelName, "hr-fernsehen", IGNORE_HD))               return true;
@@ -34,7 +34,7 @@ bool cCriteria::GoodVPS() {
     if (CompareChannelName(channelName, "NDR_FS",       IGNORE_HD | IGNORE_CITY)) return true;
     if (CompareChannelName(channelName, "rbb",          IGNORE_HD | IGNORE_CITY)) return true;
     if (CompareChannelName(channelName, "SR_Fernsehen", IGNORE_HD))               return true;
-    if (CompareChannelName(channelName, "tagesschau24", IGNORE_HD))               return true;
+//    if (CompareChannelName(channelName, "tagesschau24", IGNORE_HD))               return true;   // no good VPS events
     if (CompareChannelName(channelName, "WDR",          IGNORE_HD | IGNORE_CITY)) return true;
     if (CompareChannelName(channelName, "ZDF",          IGNORE_HD))               return true;
     if (CompareChannelName(channelName, "ZDFinfo",      IGNORE_HD))               return true;
@@ -50,7 +50,7 @@ int cCriteria::LogoFadeInOut() {
     if (CompareChannelName(channelName, "Comedy_Central", IGNORE_HD))                  return FADE_OUT;           // logo stop before broadcast end
     if (CompareChannelName(channelName, "Das_Erste",      IGNORE_HD))                  return FADE_IN | FADE_OUT;
     if (CompareChannelName(channelName, "Disney_Channel", IGNORE_HD))                  return FADE_IN | FADE_OUT;
-    if (CompareChannelName(channelName, "DMAX",           IGNORE_HD))                  return FADE_IN | FADE_OUT;
+    if (CompareChannelName(channelName, "DMAX",           IGNORE_HD | IGNORE_COUNTRY)) return FADE_IN | FADE_OUT;
     if (CompareChannelName(channelName, "FOX_Channel",    IGNORE_HD))                  return FADE_IN;
     if (CompareChannelName(channelName, "kabel_eins",     IGNORE_HD))                  return FADE_IN;
     if (CompareChannelName(channelName, "KiKA",           IGNORE_HD))                  return FADE_IN | FADE_OUT;
@@ -101,21 +101,24 @@ bool cCriteria::LogoInBorder() {
 
 bool cCriteria::NoLogo() {   // channel were logo detection is not possible
     if (!channelName) return false;
-
     if (CompareChannelName(channelName, "Sky_Cinema_Special_HD", IGNORE_NOTHING)) return true;  // channel have no continuous logo
     if (CompareChannelName(channelName, "TF1",                   IGNORE_NOTHING)) return true;  // channel have no logo
+    return false;
+}
 
+
+bool cCriteria::LogoMissingAtEndChannel() {   // channel with logo interuption short before broadcast end
+    if (!channelName) return false;
+    if (CompareChannelName(channelName, "TLC", IGNORE_HD)) return true;
     return false;
 }
 
 
 bool cCriteria::LogoInNewsTicker() {
     if (!channelName) return false;
-
     if (CompareChannelName(channelName, "n-tv", IGNORE_HD)) return true;
     if (CompareChannelName(channelName, "ntv", IGNORE_HD))  return true;
     if (CompareChannelName(channelName, "WELT", IGNORE_HD)) return true;
-
     return false;
 }
 
