@@ -129,12 +129,11 @@ cDecoder::~cDecoder() {
 #endif
         if (av_buffer_get_ref_count(hw_device_ctx) > 1) {
             isyslog("cDecoder::~cDecoder(): to much buffer references %d", av_buffer_get_ref_count(hw_device_ctx));
-            isyslog("cDecoder::~cDecoder(): FFmpeg memory leak introduced with commit 9db68ed042a9043362d57c79945f6a8d936f9dba (FFmpeg >= 7.1)");
+            isyslog("cDecoder::~cDecoder(): FFmpeg memory leak introduced with commit 9db68ed042a9043362d57c79945f6a8d936f9dba (only FFmpeg 7.1.x)");
         }
         FREE(sizeof(*hw_device_ctx), "hw_device_ctx");
         av_buffer_unref(&hw_device_ctx);  // have to unref both to reduce ref-counter
     }
-
     if (recordingDir) {
         FREE(strlen(recordingDir), "recordingDir");
         free(recordingDir);
@@ -262,6 +261,8 @@ bool cDecoder::ReadNextFile() {
             ret = InitDecoder(filename);
         }
         if (!ret) {
+            FREE(strlen(filename), "filename");
+            free(filename);
             esyslog("cDecoder:::ReadNextFile(): init decoder failed");
             return false;
         }
@@ -269,6 +270,8 @@ bool cDecoder::ReadNextFile() {
     else {
         dsyslog("cDecoder:::ReadNextFile(): next file %s does not exists", filename);
         if (fileNumber <= 1) {
+            FREE(strlen(filename), "filename");
+            free(filename);
             esyslog("cDecoder:::ReadNextFile(): file 00001.ts does not exists");
             exit(EXIT_FAILURE);
         }
@@ -733,7 +736,8 @@ bool cDecoder::ReadPacket() {
 
     frameValid = false;
     av_packet_unref(&avpkt);
-    if (av_read_frame(avctx, &avpkt) == 0 ) {
+    int av_read_rc = av_read_frame(avctx, &avpkt);
+    if (av_read_rc == 0) {
         // check packet DTS and PTS
         if (avpkt.pts == AV_NOPTS_VALUE) {
             dsyslog("cDecoder::ReadPacket(): packet (%5d), stream %d, duration %" PRId64 ": PTS not set", packetNumber, avpkt.stream_index, avpkt.duration);
@@ -822,6 +826,7 @@ bool cDecoder::ReadPacket() {
         }
         return true;
     }
+    dsyslog("cDecoder::ReadPacket(): av_read_rc = %d", av_read_rc);
 // end of file reached
     dsyslog("cDecoder::ReadPacket(): packet (%d): end of of filenumber %d ", packetNumber, fileNumber);
     if (decodeErrorFrame == packetNumber) decodeErrorCount--; // ignore malformed last frame of a file
