@@ -132,7 +132,9 @@ typedef struct sMarkAdContext {
         //!<
         int tStart                     = -1;       //!< offset of timer start to recording start (pre timer)
         //!<
-        bool startVPS                  = false;    //!< tStart is from VPS start event
+        bool startFromVPS              = false;    //!< tStart is from VPS start event
+        //!<
+        bool lengthFromVPS             = false;    //!< recording length is from "VPS stop event - VPS start" event
         //!<
         sAspectRatio AspectRatio       = {0};      //!< set from info file and checked after chkSTART, valid for the broadcast
         //!<
@@ -289,6 +291,13 @@ private:
      */
     void SwapAspectRatio();
 
+    /**
+     * check if there is a missing logo sequence at end from TLC
+     * @param mark check this mark
+     * @return true if separator found
+     */
+    bool HaveLogoMissingSequence(const cMark *mark);
+
     /**
      * check if there is a opening/closing logo sequence from kabel eins
      * @param mark check this mark
