@@ -51,13 +51,13 @@ private:
     //!<
     int port;                                        //!< VDR port number to send OSD messages
     //!<
-    char *msg     = NULL;                            //!< OSD message
+    char *msg     = nullptr;                            //!< OSD message
     //!<
     pthread_t tid = 0;                               //!< thread id of the OSD message
     //!<
     static void *SendMessage(void *posd);             //!< send OSD message
     //!<
-    static bool ReadReply(int fd, char **reply = NULL);     //!< read reply from OSD
+    static bool ReadReply(int fd, char **reply = nullptr);     //!< read reply from OSD
     //!<
 };
 #endif
