@@ -30,9 +30,6 @@ bool cOverlap::DetectOverlap(cMarks *marksParam) {
     if (!marksParam) return false;
     marks = marksParam;
 
-    LogSeparator(true);
-    dsyslog("cOverlap::DetectOverlap(): start overlap detection");
-    marks->Debug();
     if (marks->Count() < 4) {
         dsyslog("cOverlap::DetectOverlap(): not enough marks for overlap detection");
         return false;
@@ -62,7 +59,8 @@ bool cOverlap::DetectOverlap(cMarks *marksParam) {
 
             // detect overlap before stop and after start
             if (!ProcessMarksOverlap(overlapAroundAd, &p1, &p2)) {
-                dsyslog("cOverlap::DetectOverlap(): no overlap found before stop mark (%d) and after start (%d)", p1->position, p2->position);
+                if (p1 && p2) dsyslog("cOverlap::DetectOverlap(): no overlap found before stop mark (%d) and after start (%d)", p1->position, p2->position);
+                else esyslog("cOverlap::DetectOverlap(): marks invalid after ProcessMarksOverlap()");
             }
             else save = true;
 
@@ -93,15 +91,16 @@ bool cOverlap::ProcessMarksOverlap(cOverlapAroundAd *overlapAroundAd, cMark **ma
     if (!(*mark2))  return false;
 
     sOverlapPos overlapPos;
-    overlapPos.similarBeforeStart = -1;
-    overlapPos.similarBeforeEnd   = -1;
-    overlapPos.similarAfterStart  = -1;
-    overlapPos.similarAfterEnd    = -1;
+    overlapPos.similarBeforeStartPacketNumber = -1;
+    overlapPos.similarBeforeEndPacketNumber   = -1;
+    overlapPos.similarAfterStartPacketNumber  = -1;
+    overlapPos.similarAfterEndPacketNumber    = -1;
 
     int frameRate = decoder->GetVideoFrameRate();
 
-#define OVERLAP_CHECK_BEFORE 90  // start before stop mark, max found 58s, changed from 120 to 90
-#define OVERLAP_CHECK_AFTER  90  // end after start mark,                  changed from 300 to 90
+// max overlap found: 94200ms (TELE 5)
+#define OVERLAP_CHECK_BEFORE 100  // start before stop mark, changed from 90 to 100
+#define OVERLAP_CHECK_AFTER  100  // end after start mark,   changed from 90 to 100
 
     // calculate overlap check positions
     int fRangeBegin = (*mark1)->position - (frameRate * OVERLAP_CHECK_BEFORE);
@@ -125,7 +124,7 @@ bool cOverlap::ProcessMarksOverlap(cOverlapAroundAd *overlapAroundAd, cMark **ma
     }
 
     // seek to start frame of overlap check
-    char *indexToHMSF = marks->IndexToHMSF(fRangeBegin, false);
+    char *indexToHMSF = marks->IndexToHMSF(fRangeBegin, AV_NOPTS_VALUE, false);
     if (indexToHMSF) {
         ALLOC(strlen(indexToHMSF)+1, "indexToHMSF");
         dsyslog("cOverlap::ProcessMarksOverlap(): start check %ds before start mark (%d) from frame (%d) at %s", OVERLAP_CHECK_BEFORE, (*mark1)->position, fRangeBegin, indexToHMSF);
@@ -168,9 +167,9 @@ bool cOverlap::ProcessMarksOverlap(cOverlapAroundAd *overlapAroundAd, cMark **ma
         if ((decoder->GetPacketNumber() > (DEBUG_OVERLAP_FRAME_BEFORE - DEBUG_OVERLAP_FRAME_RANGE)) &&
                 (decoder->GetPacketNumber() < (DEBUG_OVERLAP_FRAME_BEFORE + DEBUG_OVERLAP_FRAME_RANGE))) SaveFrame(decoder->GetPacketNumber(), nullptr, nullptr);
 #endif
-        sVideoPicture *picture = decoder->GetVideoPicture();
+        const sVideoPicture *picture = decoder->GetVideoPicture();
         if (!picture) continue;
-        overlapAroundAd->Process(picture, decoder->GetPacketNumber(), frameCount, true, (decoder->GetVideoType() == MARKAD_PIDTYPE_VIDEO_H264));
+        overlapAroundAd->Process(picture, frameCount, true, (decoder->GetVideoType() == MARKAD_PIDTYPE_VIDEO_H264));
     }
 
     // seek to iFrame before start mark
@@ -180,7 +179,7 @@ bool cOverlap::ProcessMarksOverlap(cOverlapAroundAd *overlapAroundAd, cMark **ma
         return false;
     }
     if (fRangeBegin <  decoder->GetPacketNumber()) fRangeBegin = decoder->GetPacketNumber(); // on very short stop/start pairs we have no room to go before start mark
-    indexToHMSF = marks->IndexToHMSF(fRangeBegin, false);
+    indexToHMSF = marks->IndexToHMSF(fRangeBegin, AV_NOPTS_VALUE, false);
     if (indexToHMSF) {
         ALLOC(strlen(indexToHMSF)+1, "indexToHMSF");
         dsyslog("cOverlap::ProcessMarksOverlap(): seek forward to frame (%d) at %s before start mark (%d) and start overlap check", fRangeBegin, indexToHMSF, (*mark2)->position);
@@ -213,57 +212,57 @@ bool cOverlap::ProcessMarksOverlap(cOverlapAroundAd *overlapAroundAd, cMark **ma
                 (decoder->GetPacketNumber() < (DEBUG_OVERLAP_FRAME_AFTER + DEBUG_OVERLAP_FRAME_RANGE))) SaveFrame(decoder->GetPacketNumber(), nullptr, nullptr);
 #endif
 
-        sVideoPicture *picture = decoder->GetVideoPicture();
+        const sVideoPicture *picture = decoder->GetVideoPicture();
         if (!picture) continue;
-        overlapAroundAd->Process(picture, decoder->GetPacketNumber(), frameCount, false, (decoder->GetVideoType() == MARKAD_PIDTYPE_VIDEO_H264));
+        overlapAroundAd->Process(picture, frameCount, false, (decoder->GetVideoType() == MARKAD_PIDTYPE_VIDEO_H264));
     }
 
     dsyslog("cOverlapAroundAd::ProcessMarksOverlap(): start compare frames");
     overlapAroundAd->Detect(&overlapPos);
 #ifdef DEBUG_OVERLAP
-    dsyslog("cOverlapAroundAd::ProcessMarksOverlap(): overlap from (%d) before stop mark and (%d) after start mark", overlapPos.similarBeforeEnd, overlapPos.similarAfterEnd);
+    dsyslog("cOverlapAroundAd::ProcessMarksOverlap(): overlap from (%d) before stop mark and (%d) after start mark", overlapPos.similarBeforeEndPacketNumber, overlapPos.similarAfterEndPacketNumber);
 #endif
-    if (overlapPos.similarAfterEnd >= 0) {
+    if (overlapPos.similarAfterEndPacketNumber >= 0) {
         // found overlap
-        int lengthBefore = 1000 * (overlapPos.similarBeforeEnd - overlapPos.similarBeforeStart + 1) / frameRate; // include first and last
-        int lengthAfter  = 1000 * (overlapPos.similarAfterEnd  - overlapPos.similarAfterStart + 1)  / frameRate;
+        int lengthBefore = 1000 * (overlapPos.similarBeforeEndPacketNumber - overlapPos.similarBeforeStartPacketNumber + 1) / frameRate; // include first and last
+        int lengthAfter  = 1000 * (overlapPos.similarAfterEndPacketNumber  - overlapPos.similarAfterStartPacketNumber  + 1) / frameRate;
 
-        char *indexToHMSFbeforeStart = marks->IndexToHMSF(overlapPos.similarBeforeStart, false);
+        char *indexToHMSFbeforeStart = marks->IndexToHMSF(overlapPos.similarBeforeStartPacketNumber, overlapPos.similarBeforeStartPTS, false);
         if (indexToHMSFbeforeStart) {
             ALLOC(strlen(indexToHMSFbeforeStart)+1, "indexToHMSFbeforeStart");
         }
 
-        char *indexToHMSFbeforeEnd   = marks->IndexToHMSF(overlapPos.similarBeforeEnd, false);
+        char *indexToHMSFbeforeEnd   = marks->IndexToHMSF(overlapPos.similarBeforeEndPacketNumber, overlapPos.similarBeforeEndPTS, false);
         if (indexToHMSFbeforeEnd) {
             ALLOC(strlen(indexToHMSFbeforeEnd)+1, "indexToHMSFbeforeEnd");
         }
 
-        char *indexToHMSFafterStart  = marks->IndexToHMSF(overlapPos.similarAfterStart, false);
+        char *indexToHMSFafterStart  = marks->IndexToHMSF(overlapPos.similarAfterStartPacketNumber, overlapPos.similarAfterStartPTS, false);
         if (indexToHMSFafterStart) {
             ALLOC(strlen(indexToHMSFafterStart)+1, "indexToHMSFafterStart");
         }
 
-        char *indexToHMSFafterEnd    = marks->IndexToHMSF(overlapPos.similarAfterEnd, false);
+        char *indexToHMSFafterEnd    = marks->IndexToHMSF(overlapPos.similarAfterEndPacketNumber, overlapPos.similarAfterEndPTS, false);
         if (indexToHMSFafterEnd) {
             ALLOC(strlen(indexToHMSFafterEnd)+1, "indexToHMSFafterEnd");
         }
 
-        dsyslog("cOverlap::ProcessMarksOverlap(): similar from (%5d) at %s to (%5d) at %s, length %5dms", overlapPos.similarBeforeStart, indexToHMSFbeforeStart, overlapPos.similarBeforeEnd, indexToHMSFbeforeEnd, lengthBefore);
-        dsyslog("cOverlap::ProcessMarksOverlap():              (%5d) at %s to (%5d) at %s, length %5dms",     overlapPos.similarAfterStart,  indexToHMSFafterStart,  overlapPos.similarAfterEnd,  indexToHMSFafterEnd, lengthAfter);
+        dsyslog("cOverlap::ProcessMarksOverlap(): similar from (%5d) PTS %" PRId64 " at %s to (%5d) PTS %" PRId64 " at %s, length %5dms", overlapPos.similarBeforeStartPacketNumber, overlapPos.similarBeforeStartPTS, indexToHMSFbeforeStart, overlapPos.similarBeforeEndPacketNumber, overlapPos.similarBeforeEndPTS, indexToHMSFbeforeEnd, lengthBefore);
+        dsyslog("cOverlap::ProcessMarksOverlap():              (%5d) at %s to (%5d) at %s, length %5dms",     overlapPos.similarAfterStartPacketNumber,  indexToHMSFafterStart,  overlapPos.similarAfterEndPacketNumber,  indexToHMSFafterEnd, lengthAfter);
         dsyslog("cOverlap::ProcessMarksOverlap():              maximum deviation in overlap %6d", overlapPos.similarMax);
         if (overlapPos.similarEnd > 0) dsyslog("cOverlap::ProcessMarksOverlap():              next deviation after overlap %6d", overlapPos.similarEnd); // can be 0 if overlap ends at the mark
 
         const char *indexToHMSFmark1  = marks->GetTime(*mark1);
         const char *indexToHMSFmark2  = marks->GetTime(*mark2);
 
-        int gapStop          = ((*mark1)->position - overlapPos.similarBeforeEnd)   / frameRate;
-        int lengthBeforeStop = ((*mark1)->position - overlapPos.similarBeforeStart) / frameRate;
-        int gapStart         = (overlapPos.similarAfterStart - (*mark2)->position)  / frameRate;
-        int lengthAfterStart = (overlapPos.similarAfterEnd - (*mark2)->position)    / frameRate;
+        int gapStop          = ((*mark1)->position - overlapPos.similarBeforeEndPacketNumber)   / frameRate;
+        int lengthBeforeStop = ((*mark1)->position - overlapPos.similarBeforeStartPacketNumber) / frameRate;
+        int gapStart         = (overlapPos.similarAfterStartPacketNumber - (*mark2)->position)  / frameRate;
+        int lengthAfterStart = (overlapPos.similarAfterEndPacketNumber - (*mark2)->position)    / frameRate;
 
         if (indexToHMSFbeforeStart && indexToHMSFbeforeEnd && indexToHMSFafterStart && indexToHMSFafterEnd && indexToHMSFmark1 && indexToHMSFmark2) {
-            dsyslog("cOverlap::ProcessMarksOverlap(): overlap from (%6d) at %s to (%6d) at %s, before stop mark gap %3ds length %3ds, are identical with",overlapPos.similarBeforeEnd, indexToHMSFbeforeEnd, (*mark1)->position, indexToHMSFmark1, gapStop, lengthBeforeStop);
-            dsyslog("cOverlap::ProcessMarksOverlap():              (%6d) at %s to (%6d) at %s, after start mark gap %3ds length %3ds", (*mark2)->position, indexToHMSFmark2, overlapPos.similarAfterEnd, indexToHMSFafterEnd, gapStart, lengthAfterStart);
+            dsyslog("cOverlap::ProcessMarksOverlap(): overlap from (%6d) at %s to (%6d) at %s, before stop mark gap %3ds length %3ds, are identical with",overlapPos.similarBeforeEndPacketNumber, indexToHMSFbeforeEnd, (*mark1)->position, indexToHMSFmark1, gapStop, lengthBeforeStop);
+            dsyslog("cOverlap::ProcessMarksOverlap():              (%6d) at %s to (%6d) at %s, after start mark gap %3ds length %3ds", (*mark2)->position, indexToHMSFmark2, overlapPos.similarAfterEndPacketNumber, indexToHMSFafterEnd, gapStart, lengthAfterStart);
         }
         if (indexToHMSFbeforeStart) {
             FREE(strlen(indexToHMSFbeforeStart)+1, "indexToHMSFbeforeStart");
@@ -294,14 +293,12 @@ bool cOverlap::ProcessMarksOverlap(cOverlapAroundAd *overlapAroundAd, cMark **ma
             // but if it is too far away it is a false positiv
             // changed gapStop from 36 to 27
             dsyslog("cOverlap::ProcessMarksOverlap(): overlap gap to marks are valid, before stop mark %ds, after start mark %ds, length %dms", gapStop, gapStart, lengthBefore);
-            int offset = overlapPos.similarBeforeEnd - (*mark1)->position;
-            *mark1 = marks->Move((*mark1), *mark1, offset, MT_OVERLAPSTOP);
+            *mark1 = marks->Move((*mark1), overlapPos.similarBeforeEndPacketNumber, overlapPos.similarBeforeEndPTS, MT_OVERLAPSTOP);
             if (!(*mark1)) {
                 esyslog("cOverlap::ProcessMarksOverlap(): move stop mark failed");
                 return false;
             }
-            offset = overlapPos.similarAfterEnd - (*mark2)->position;
-            *mark2 = marks->Move((*mark2), (*mark2), offset, MT_OVERLAPSTART);
+            *mark2 = marks->Move((*mark2), overlapPos.similarAfterEndPacketNumber, overlapPos.similarAfterEndPTS, MT_OVERLAPSTART);
             if (!(*mark2)) {
                 esyslog("cOverlap::ProcessMarksOverlap(): move start mark failed");
                 return false;
@@ -341,7 +338,7 @@ cOverlapAroundAd::~cOverlapAroundAd() {
     }
 }
 
-void cOverlapAroundAd::Process(sVideoPicture *picture, const int frameNumber, const int frameCount, const bool beforeAd, const bool h264) {
+void cOverlapAroundAd::Process(const sVideoPicture *picture, const int frameCount, const bool beforeAd, const bool h264) {
 #ifdef DEBUG_OVERLAP
     dsyslog("cOverlapAroundAd::Process(): frameNumber %d, frameCount %d, beforeAd %d, isH264 %d", frameNumber, frameCount, beforeAd, h264);
 #endif
@@ -370,7 +367,8 @@ void cOverlapAroundAd::Process(sVideoPicture *picture, const int frameNumber, co
         }
         GetHistogram(picture, histbuf[OV_BEFORE][histcnt[OV_BEFORE]].histogram);
         histbuf[OV_BEFORE][histcnt[OV_BEFORE]].valid = true;
-        histbuf[OV_BEFORE][histcnt[OV_BEFORE]].frameNumber = frameNumber;
+        histbuf[OV_BEFORE][histcnt[OV_BEFORE]].frameNumber = picture->packetNumber;
+        histbuf[OV_BEFORE][histcnt[OV_BEFORE]].pts         = picture->pts;
         histcnt[OV_BEFORE]++;
     }
     else {
@@ -391,10 +389,11 @@ void cOverlapAroundAd::Process(sVideoPicture *picture, const int frameNumber, co
         }
         GetHistogram(picture, histbuf[OV_AFTER][histcnt[OV_AFTER]].histogram);
         histbuf[OV_AFTER][histcnt[OV_AFTER]].valid = true;
-        histbuf[OV_AFTER][histcnt[OV_AFTER]].frameNumber = frameNumber;
+        histbuf[OV_AFTER][histcnt[OV_AFTER]].frameNumber = picture->packetNumber;
+        histbuf[OV_AFTER][histcnt[OV_AFTER]].pts         = picture->pts;
         histcnt[OV_AFTER]++;
     }
-    lastFrameNumber = frameNumber;
+    lastFrameNumber = picture->packetNumber;
     return;
 }
 
@@ -464,12 +463,16 @@ void cOverlapAroundAd::Detect(sOverlapPos *overlapPos) {
             // found long enough overlap, store position
 
             if ((simLength >= similarMinLength) &&
-                    ((histbuf[OV_BEFORE][tmpindexBeforeStopMark].frameNumber - histbuf[OV_BEFORE][firstSimilarBeforeStopMark].frameNumber) >= (overlapPos->similarBeforeEnd - overlapPos->similarBeforeStart))) { // new overlap is longer than current overlap
-                overlapPos->similarBeforeStart = histbuf[OV_BEFORE][firstSimilarBeforeStopMark].frameNumber;
-                overlapPos->similarBeforeEnd   = histbuf[OV_BEFORE][tmpindexBeforeStopMark].frameNumber;
-                overlapPos->similarAfterStart  = histbuf[OV_AFTER][firstSimilarAfterStartMark].frameNumber;
-                overlapPos->similarAfterEnd    = histbuf[OV_AFTER][tmpindexAfterStartMark].frameNumber;
-                overlapPos->similarMax         = simMax;
+                    ((histbuf[OV_BEFORE][tmpindexBeforeStopMark].frameNumber - histbuf[OV_BEFORE][firstSimilarBeforeStopMark].frameNumber) >= (overlapPos->similarBeforeEndPacketNumber - overlapPos->similarBeforeStartPacketNumber))) { // new overlap is longer than current overlap
+                overlapPos->similarBeforeStartPacketNumber = histbuf[OV_BEFORE][firstSimilarBeforeStopMark].frameNumber;
+                overlapPos->similarBeforeStartPTS          = histbuf[OV_BEFORE][firstSimilarBeforeStopMark].pts;
+                overlapPos->similarBeforeEndPacketNumber   = histbuf[OV_BEFORE][tmpindexBeforeStopMark].frameNumber;
+                overlapPos->similarBeforeEndPTS            = histbuf[OV_BEFORE][tmpindexBeforeStopMark].pts;
+                overlapPos->similarAfterStartPacketNumber  = histbuf[OV_AFTER][firstSimilarAfterStartMark].frameNumber;
+                overlapPos->similarAfterStartPTS           = histbuf[OV_AFTER][firstSimilarAfterStartMark].pts;
+                overlapPos->similarAfterEndPacketNumber    = histbuf[OV_AFTER][tmpindexAfterStartMark].frameNumber;
+                overlapPos->similarAfterEndPTS             = histbuf[OV_AFTER][tmpindexAfterStartMark].pts;
+                overlapPos->similarMax                     = simMax;
                 if (simil < 0) overlapPos->similarEnd = -simil;
             }
 
@@ -514,7 +517,7 @@ void cOverlapAroundAd::Detect(sOverlapPos *overlapPos) {
 }
 
 
-void cOverlapAroundAd::GetHistogram(sVideoPicture *picture, simpleHistogram &dest) const {
+void cOverlapAroundAd::GetHistogram(const sVideoPicture *picture, simpleHistogram &dest) const {
     memset(dest, 0, sizeof(simpleHistogram));
     int videoHeight = decoder->GetVideoHeight();
     int videoWidth  = decoder->GetVideoWidth();
